package li.klass.confluence.tableagg.plugin

import org.specs.SpecificationWithJUnit
import TableAggregationMacro._

class TableAggregationMacroTest extends SpecificationWithJUnit {
  "test horizontal aggregation" in {
    val content = """
      <table><tbody><tr><th>&nbsp;</th><th>&nbsp;</th><th>&nbsp;</th></tr><tr><td>1</td><td>1</td><td><ac:macro ac:name="tableagg"><ac:parameter ac:name="orientation">horizontal</ac:parameter><ac:parameter ac:name="id">id</ac:parameter></ac:macro></td></tr><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr></tbody></table>
      <table><tbody><tr><th>&nbsp;</th><th>&nbsp;</th><th>&nbsp;</th></tr><tr><td>200</td><td>3</td><td><ac:macro ac:name="tableagg"><ac:parameter ac:name="orientation">horizontal</ac:parameter><ac:parameter ac:name="id">id1</ac:parameter></ac:macro></td></tr><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr></tbody></table>
      <table><tbody><tr><th>&nbsp;</th><th>&nbsp;</th><th>&nbsp;</th></tr><tr><td>abc</td><td>3</td><td><ac:macro ac:name="tableagg"><ac:parameter ac:name="orientation">horizontal</ac:parameter><ac:parameter ac:name="id">id3</ac:parameter></ac:macro></td></tr><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr><tr><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td></tr></tbody></table>
    """

    TableAggregationMacro.aggregate(content, "id", 0, findRowForId, _ + _) must_== 2
    TableAggregationMacro.aggregate(content, "id1", 0, findRowForId, _ + _) must_== 203
    TableAggregationMacro.aggregate(content, "id2", 0, findRowForId, _ + _) must_== -1
    TableAggregationMacro.aggregate(content, "id3", 0, findRowForId, _ + _) must_== 3
  }

  "test vertical aggregation" in {
    val content = """
      <table><tbody><tr><th>a</th><th>bä</th></tr><tr><td>1</td><td>2</td></tr><tr><td>3</td><td>5</td></tr><tr>
      <td><ac:macro ac:name="tableagg"><ac:parameter ac:name="id">3</ac:parameter><ac:parameter ac:name="orientation">vertical</ac:parameter></ac:macro></td>
      <td><ac:macro ac:name="tableagg"><ac:parameter ac:name="id">4</ac:parameter><ac:parameter ac:name="orientation">vertical</ac:parameter></ac:macro></td></tr></tbody></table>
    """

    TableAggregationMacro.aggregate(content, "3", 0, findColumnForId, _ + _) must_== 4
    TableAggregationMacro.aggregate(content, "4", 0, findColumnForId, _ + _) must_== 7
  }

  "test horizontal aggregation" in {
    val content = """
    <table>
    <tbody>
    <tr>
    <td>&auml;&ouml;&Ouml;&szlig;</td>
    <td>1</td>
    <td>1</td>
    <td><strong><ac:macro ac:name="tableagg"><ac:parameter ac:name="id">1</ac:parameter><ac:parameter ac:name="aggregation">sum</ac:parameter><ac:parameter ac:name="orientation">horizontal</ac:parameter></ac:macro></strong></td></tr>
    <tr>
    <td>äüß</td>
    <td>3</td>
    <td>6</td>
    <td><strong><ac:macro ac:name="tableagg"><ac:parameter ac:name="id">2</ac:parameter><ac:parameter ac:name="aggregation">sum</ac:parameter><ac:parameter ac:name="orientation">horizontal</ac:parameter></ac:macro></strong></td></tr></tbody></table>
    """
    TableAggregationMacro.aggregate(content, "1", 0, findRowForId, _ + _) must_== 2
    TableAggregationMacro.aggregate(content, "2", 1, findRowForId, _ * _) must_== 18
  }

  "test strange macro error" in {
    val content = """
      <table>
      <tbody>
      <tr>
      <th>Struktur</th>
      <th colspan="1">Datum</th>
      <th>Aufgaben</th>
      <th>Aufwand (d)</th></tr>
      <tr>
      <td colspan="1">Projektstart</td>
      <td colspan="1">&nbsp;</td>
      <td colspan="1">&nbsp;</td>
      <td colspan="1">&nbsp;</td></tr>
      <tr>
      <td colspan="1">&nbsp;</td>
      <td colspan="1">02.04.2012</td>
      <td colspan="1">
      <ul>
      <li>Einrichtung Serverinfrastruktur (Confluence, Jira, Jenkins, Git)</li>
      <li>Jira Workflow</li></ul></td>
      <td colspan="1" style="text-align: center;">0.5</td></tr>
      <tr>
      <td colspan="1">&nbsp;</td>
      <td colspan="1">03.04.2012</td>
      <td colspan="1">
      <ul>
      <li>Installation Nexus</li>
      <li>Jenkins Entwicklungsflow</li></ul></td>
      <td colspan="1" style="text-align: center;">0.5</td></tr>
      <tr>
      <td colspan="1">&nbsp;</td>
      <td colspan="1">
      <p>04.04.2012<br />05.04.2012&nbsp;</p></td>
      <td colspan="1">
      <ul>
      <li>Umzug Wiki Stromb&ouml;rse (Coding Conventions, DoD)</li>
      <li><span style="color: rgb(0,51,102);"><ac:link><ri:page ri:content-title="Embedding Semantic Product Memories in the web of things" /></ac:link><br /></span></li></ul></td>
      <td colspan="1" style="text-align: center;">0.5</td></tr>
      <tr>
      <td class="highlight" colspan="1">&nbsp;</td>
      <td class="highlight" colspan="1">&nbsp;</td>
      <td class="highlight" colspan="1">&nbsp;</td>
      <td class="highlight" colspan="1" style="text-align: center;"><ac:macro ac:name="tableagg"><ac:parameter ac:name="id">aufwandsum</ac:parameter><ac:parameter ac:name="aggregation">sum</ac:parameter><ac:parameter ac:name="orientation">vertical</ac:parameter></ac:macro></td></tr></tbody></table>
    """
    TableAggregationMacro.aggregate(content, "aufwandsum", 0, findColumnForId, _ + _) must_== 1.5
  }
}
